import sensor, image, time, gc, math, support_func

def calibrate_sensor():
    sensor.reset()
    sensor.set_pixformat(sensor.GRAYSCALE)
    sensor.set_framesize(sensor.VGA)
    sensor.set_auto_gain(True)
    sensor.set_gainceiling(2)
    sensor.set_auto_exposure(True)
    sensor.set_auto_whitebal(True)
    sensor.set_contrast(1)
    sensor.set_brightness(3)
    sensor.skip_frames(time = 2000)


def get_lines(params, pos):

    calibrate_sensor()
    lines = []
    img = sensor.snapshot()                        # Take picture
    width_sections = int(img.width() / 80)         # Segment image in x axis
    height_sections = int(img.height() / 60)       # Segment image in y axis
    rest_width = int(img.width() % 80)             # Find leftover pixels
    rest_height = int(img.height() % 60)

    gc.collect()                                  # Clear memory heap

    # Finds line segments for each image segment, adds to cumulative list
    for i in range(width_sections):
        for j in range(height_sections):
            block_lines = img.find_line_segments(roi = [i*80,j*60,80,60],merge_distance = params[0],
                                                 max_theta_difference = params[1])
            for l in block_lines:
                lines.append(l)
            del block_lines
            gc.collect()

            if i == (width_sections-1) and (rest_width != 0):
                block_lines = img.find_line_segments(roi = [width_sections*80,j*60,rest_width,rest_height],
                                                     merge_distance = params[0], max_theta_difference = params[1])
                for l in block_lines:
                    lines.append(l)
                del block_lines
                gc.collect()

            if j == (height_sections-1) and (rest_height != 0):
                block_lines =img.find_line_segments(roi = [i*80,height_sections*60,rest_width,rest_height],
                                                    merge_distance = params[0], max_theta_difference = params[1])
                for l in block_lines:
                    lines.append(l)
                del block_lines
                gc.collect()

    filter_lines = []
    # Filter out lines that are weak or too short
    for l in lines:
        if l.length() > params[2] and l.magnitude() > params[3]:
            filter_lines.append(l)
            img.draw_line(l.line(), color = (255, 255, 255))         # Draws lines before merging (white)

    merged_lines = support_func.merge_lines(params, filter_lines)    # Matches and merges lines
    print("Lines before merging: ", len(filter_lines))
    print("Lines after merging:  ", len(merged_lines))
    temp = []
    for l in merged_lines:
        if l.length() > params[4]:                                   # Filter and draw merged lines (black)
            img.draw_line(l.get_cord(), color = (0, 0, 0))
            real_line = support_func.calculate_distance(l, pos, [img.width(), img.height()])
            print("PAIR:")
            l.print()
            real_line.print()
            print("\n")
            temp.append(real_line)
    # Draw center of image (does not correspond with sensor center)
    img.draw_line(312,237,320,237, color = (255,255,255))
    img.draw_line(316,233,316,241, color = (255,255,255))

    # Visualize image in IDE
    print(img.compress_for_ide(), end="")

    del img,lines, filter_lines, params, merged_lines
    gc.collect()
    return temp

