"SPI functionality based on the file arduino_spi_slave_1.py found in the"
"examples/interface folder of openMV IDE"

import pyb, ustruct, time, get_lines, gc, math

# SPI setup
text = "Hello World!\n"
data = [ustruct.pack("<%ds" % len(text), text)]
recv_msg = ustruct.pack("<%ds" % len("REC"), "REC")
buff = bytearray(30)
capture = False
data_ready = 1
data = []
garbage_collect = 0
pos_arr = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]

# Defining tuning variables
max_theta_difference = 5
merge_distance = 10
prune_line_length = 5
prune_line_mag = 1
prune_final_length = 100
theta_diff = 6
dist_diff = 20
bound = 4

spi = pyb.SPI(2, pyb.SPI.SLAVE, polarity=0, phase=0)

def nss_callback(line):
    global spi, data, buff, read_data, capture, data_ready, recv_msg, garbage_collect, pos_arr

    if (data_ready == 0):
        try:
            spi.recv(buff, timeout = 5000)
            if(chr(buff[0]) == "C" and chr(buff[1]) == "A" and chr(buff[2]) == "P"):
                print("Recv data", buff)
                for i in range(26):
                    pos_arr[i] = buff[i+3]
                spi.send(recv_msg, timeout=1000)
                test.disable()
                capture = True
        except OSError as err:
            print("Failed receive")
            pass
    elif (data_ready == 1):
        try:
            for s in data:
                time.sleep_ms(100)
                spi.send(s, timeout=1000)
                print("Sending data: ", s)
            time.sleep_ms(1000)
            data_ready = 0
            garbage_collect = 1
        except OSError as err:
            print("Failed send")
            pass

test = pyb.ExtInt(pyb.Pin("P3"), pyb.ExtInt.IRQ_FALLING, pyb.Pin.PULL_UP, nss_callback)

while(True):
    print("Waiting... ", data_ready)
    time.sleep_ms(2000)
    spi = pyb.SPI(2, pyb.SPI.SLAVE, polarity=0, phase=0)

    if (capture == True):
        if(garbage_collect == 1):
            del data
            gc.collect()
            garbage_collect =0

        print("Processing image...")

        pos_str = ""
        print(pos_arr)
        for b in pos_arr:
            pos_str = pos_str + chr(b)
        pos_str = pos_str.split(",")
        print(pos_str)
        theta = int(pos_str[3])
        sensor_off = 45.5 # cm
        cam_offset = [math.sin((-theta*math.pi)/ 180)*sensor_off, math.cos((-theta*math.pi)/ 180)*sensor_off]

        pos = (int(pos_str[0])+cam_offset[0], int(pos_str[1])+cam_offset[1], int(pos_str[2]), (theta*math.pi)/ 180)
        print(pos)
        params = [max_theta_difference, merge_distance, prune_line_length,
          prune_line_mag, prune_final_length, theta_diff, dist_diff,
           bound]

        lines = get_lines.get_lines(params, pos)

        num_lines = str(len(lines))
        data = [("LIN:"+ str(len(num_lines)) + ":" + num_lines + "\n")]

        for l in lines:
            l_send = str(int(l.x_1))+","+str(int(l.y_1))+","+ str(int(l.x_2))+","+str(int(l.y_2))+","+str(int(l.theta)) + ",\n"
            l_string = ustruct.pack("<%ds" % len(l_send), l_send)
            data.append(l_string)
            del l_send, l_string
        capture = False
        data_ready = 1
        del lines, params, pos_str, pos
        gc.collect()
        test.enable()

